#include "tablero.h"

Tablero::Tablero(int ancho, int alto)
{
	if(ancho >= 30)       anchoTablero = 30;
	else if(ancho <= 10)  anchoTablero = 10;
	else                  anchoTablero = ancho;
	
	if(alto >= 10)      altoTablero = 10;
	else if(alto <= 5)  altoTablero = 5;
	else                altoTablero = alto;
	
	posJugador.x = 0;
	posJugador.y = altoTablero/2;
	
	nivel = 1;
	puntuacion = 0;
	jugador = "";
	
	perdido = victoria = top = challenge = false;
	
	tablero.resize(anchoTablero);
	
	for(int i=0;i<anchoTablero;i++)
		tablero[i].resize(altoTablero);
	
	for(int i=0;i<anchoTablero;i++)
		for(int j=0;j<altoTablero;j++)
			tablero[i][j].bomba = tablero[i][j].pisada = false;
	
	for(int i=0;i<nivel*5+15;i++)
	{
		int posX = rand() % anchoTablero;
		int posY = rand() % altoTablero;
		tablero[posX][posY].bomba = true;
	}
	
	tablero[posJugador.x][posJugador.y].pisada = true;
	tablero[posJugador.x][posJugador.y].bomba = false; // Que no empiece encima de una bomba
	
	struct timeval tim;
	gettimeofday(&tim, NULL);
	inicio = tim.tv_sec;
	
	transcurrido = 0.0;
}


void
Tablero::reiniciar(bool reiniciarPuntuacion)
{
	posJugador.x = 0;
	posJugador.y = altoTablero/2;
	
	perdido = victoria = top = challenge = false;
	
	if(reiniciarPuntuacion)
		puntuacion = 0;
	
	for(int i=0;i<anchoTablero;i++)
		for(int j=0;j<altoTablero;j++)
			tablero[i][j].bomba = tablero[i][j].pisada = false;
	
	for(int i=0;i<nivel*5+15;i++)
	{
		int posX = rand() % anchoTablero;
		int posY = rand() % altoTablero;
		tablero[posX][posY].bomba = true;
	}
	
	tablero[posJugador.x][posJugador.y].pisada = true;
	tablero[posJugador.x][posJugador.y].bomba = false; // Que no empiece encima de una bomba
	
	struct timeval tim;
	gettimeofday(&tim, NULL);
	inicio = tim.tv_sec;
	
	transcurrido = 0.0;
}

int 
Tablero::ejecutar(Ranking &ranking, Challenges &challenges)
{
	int retorno = EJUEGO;
	StdPijo& stdp = StdPijo::p();
	
	if(!perdido && !victoria)
	{
		struct timeval tim;
		
		gettimeofday(&tim, NULL);
		transcurrido = tim.tv_sec - inicio;
		
		switch(stdp.getKey())
		{
			case 'w':
				if(posJugador.y-1 >= 0)
					posJugador.y--;
				break;
			case 'a':
				if(posJugador.x-1 >= 0)
					posJugador.x--;
				break;
			case 's':
				if(posJugador.y+1 < altoTablero)
					posJugador.y++;
				break;
			case 'd':
				if(posJugador.x+1 < anchoTablero)
					posJugador.x++;
				break;
		}
		
		if(!tablero[posJugador.x][posJugador.y].pisada)
		{
			tablero[posJugador.x][posJugador.y].pisada = true;
			
			if(!tablero[posJugador.x][posJugador.y].bomba)
				puntuacion += 5;
		}
		
		if(tablero[posJugador.x][posJugador.y].bomba)
		{
			perdido = true;
			if(ranking.top(puntuacion, jugador))
				top = true;
		}
		else if (posJugador.x == anchoTablero-1)
		{
			victoria = true;
			puntuacion = puntuacion + nivel * 1000 - transcurrido * nivel * 10;
			
			if(nivel == 1)
				challenge = challenges.conseguido(0);
			if(nivel == 3)
				challenge = challenges.conseguido(1);
			if(nivel == 5)
				challenge = challenges.conseguido(2);
			if(transcurrido <= 30)
				challenge = challenges.conseguido(3);
			if(transcurrido <= 10)
				challenge = challenges.conseguido(4);
			if(puntuacion >= 16000)
				challenge = challenges.conseguido(5);
		}
	}
	
	else // Perdido o Victoria es true
	{
		char c = stdp.getKey();
		if(perdido) // Si ha perdido
		{
			if((c == KESCAPE) || (c == ' '))
				retorno = EPERDIDO;
		}
		else // Si ha ganado
		{
			if((c == KESCAPE) || (c == ' '))
				retorno = EVICTORIA;
		}
	}
	return retorno;
}

int
Tablero::bombasAlrededor(int x, int y)
{
	int bombas = 0;
	
	if(x-1 >= 0)
		if(tablero[x-1][y].bomba)
			bombas++;
		
	if(x+1 < anchoTablero)
		if(tablero[x+1][y].bomba)
			bombas++;
		
	if(y-1 >= 0)
		if(tablero[x][y-1].bomba)
			bombas++;
		
	if(y+1 < altoTablero)
		if(tablero[x][y+1].bomba)
			bombas++;
		
	return bombas;
}

void 
Tablero::mostrar()
{
	StdPijo& stdp = StdPijo::p();
	
	int ancho = anchoTablero*2+1;
	int alto = altoTablero*2+1;
	
	stdp.clearScreen();
	
	stdp.box(0,0,80,24);
	
	stdp.box(ancho+2,1,9,3);
	mvprintw(1,ancho+4,"MINAS");
	mvprintw(2,ancho+6,"%i",bombasAlrededor(posJugador.x,posJugador.y));
	
	stdp.box(ancho+2,4,10,3);
	mvprintw(4,ancho+4,"TIEMPO");
	mvprintw(5,ancho+6,"%i",transcurrido);
	
	stdp.box(ancho+2,7,9,3);
	mvprintw(7,ancho+4,"NIVEL");
	mvprintw(8,ancho+6,"%i",nivel);
	
	stdp.box(ancho+2,10,9,3);
	mvprintw(10,ancho+4,"SCORE");
	mvprintw(11,ancho+5,"%i",puntuacion);

	// Primera línea
	mvaddch(1,1,ACS_ULCORNER);
	for(int i=2;i<ancho;i++)
	{
		if(i%2) mvaddch(1,i,ACS_TTEE);
		else mvaddch(1,i,ACS_HLINE);
	}
	mvaddch(1,ancho,ACS_URCORNER);
	
	// Bucle de las líneas intermedias
	for(int i=2;i<alto;i++)
	{
		if(i%2) 
		{ 
			for(int j=1;j<=ancho;j++)
			{
				if(j==1) 
					mvaddch(i,j,ACS_LTEE);
				else if(j==ancho) 
					mvaddch(i,j,ACS_RTEE);
				else if(j%2)
					mvaddch(i,j,ACS_PLUS); 
				else if((j+1)%2)
					mvaddch(i,j,ACS_HLINE);
			}
		}
		else 
		{ 
			for(int j=1;j<=ancho;j=j+2)
				mvaddch(i,j,ACS_VLINE);
			
		}
	}
	
	//Última línea de la tabla
	mvaddch(alto,1,ACS_LLCORNER);
	for(int i=2;i<ancho;i++)
	{
		if(i%2) mvaddch(alto,i,ACS_BTEE);
		else mvaddch(alto,i,ACS_HLINE);
	}
	mvaddch(alto,ancho,ACS_LRCORNER);
	
	// Colorear casillas pisadas
	stdp.setColor(ANORMAL,CGREEN,CBLACK);
	
	for(int i=0;i<anchoTablero;i++)
		for(int j=0;j<altoTablero;j++)
			if(tablero[i][j].pisada)
				mvaddch(j*2+2,i*2+2,ACS_CKBOARD); // ACS_CKBOARD
	
	stdp.setColor(ANORMAL,CWHITE,CBLACK);
	// Mostrar posición jugador
	mvaddch(posJugador.y*2+2,posJugador.x*2+2,'@');
	
	// Si se ha perdido
	if(perdido)
	{
		mvprintw(14,ancho+3,"Perdiste");
			
		if(top)	
		{	
			if(challenge)
				mvprintw(22,2,"¡Ranking y Challenges, estás on fire! ESC o SPACEBAR para continuar");
			else
				mvprintw(22,2,"¡Has entrado en el Ranking! ESC o SPACEBAR para continuar");
		}	
		else 
		{
			if(challenge)
				mvprintw(22,2,"¡Has desbloqueado Challenge(s)! ESC o SPACEBAR para continuar");
			else
				mvprintw(22,2,"Pulse ESC o SPACEBAR para continuar");
		}
		
		stdp.setColor(ABLINK,CRED,CBLACK);
		for(int i=0;i<anchoTablero;i++)
			for(int j=0;j<altoTablero;j++)
				if(tablero[i][j].bomba)
					mvaddch(j*2+2,i*2+2,'X');
		stdp.setColor(ANORMAL,CWHITE,CBLACK);
		
		stdp.setColor(ANORMAL,CBLACK,CYELLOW);
		mvprintw(15,ancho+3,"    _____    ");
		mvprintw(16,ancho+3,"  .'     '.  ");
		mvprintw(17,ancho+3," /  x   x  \\ ");
		mvprintw(18,ancho+3,"|           |");
		mvprintw(19,ancho+3,"|   .---.   |");
		mvprintw(20,ancho+3," \\ /     \\ / ");
		mvprintw(21,ancho+3,"  '._____.'  ");
		stdp.setColor(ANORMAL,CWHITE,CBLACK);
	}
	// Si se ha ganado, se muestra un cartelito de victoria
	if(victoria)
	{
		
		if(challenge)
			mvprintw(22,2,"¡Has desbloqueado Challenge(s)! ESC o SPACEBAR para continuar");
		else
			mvprintw(22,2,"Pulse ESC o SPACEBAR para continuar");
		
		mvprintw(14,ancho+3,"Victoria");
	
		stdp.setColor(ABLINK,CRED,CBLACK);
		for(int i=0;i<anchoTablero;i++)
			for(int j=0;j<altoTablero;j++)
				if(tablero[i][j].bomba)
					mvaddch(j*2+2,i*2+2,'X');
		stdp.setColor(ANORMAL,CWHITE,CBLACK);
		
		stdp.setColor(ANORMAL,CBLACK,CYELLOW);
		mvprintw(15,ancho+3,"    _____    ");
		mvprintw(16,ancho+3,"  .'     '.  ");
		mvprintw(17,ancho+3," /  o   o  \\ ");
		mvprintw(18,ancho+3,"|           |");
		mvprintw(19,ancho+3,"|  \\     /  |");
		mvprintw(20,ancho+3," \\  '---'  / ");
		mvprintw(21,ancho+3,"  '._____.'  ");
		stdp.setColor(ANORMAL,CWHITE,CBLACK);
	}
}
